package com.tianyi.sync.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.tianyi.sync.mapper.master.LiabilitiesDataSyncMapper;
import com.tianyi.sync.mapper.slave.LiabilitiesSlaveMapper;
import com.tianyi.sync.model.*;
import com.tianyi.sync.service.LiabilitiesDataSyncService;
import com.tianyi.sync.utils.DateUtils;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StopWatch;

import java.lang.reflect.Field;
import java.util.LinkedList;
import java.util.List;

/**
 * @author 李天壮
 * @date 2024年07月30日 08:48
 */
@Service
public class LiabilitiesDataSyncServiceImpl implements LiabilitiesDataSyncService {
    private final static Logger logger = LoggerFactory.getLogger(LiabilitiesDataSyncServiceImpl.class);

    private final LiabilitiesSlaveMapper liabilitiesSlaveMapper;

    private final LiabilitiesDataSyncMapper liabilitiesDataSyncMapper;

    public LiabilitiesDataSyncServiceImpl(LiabilitiesSlaveMapper liabilitiesSlaveMapper, LiabilitiesDataSyncMapper liabilitiesDataSyncMapper) {
        this.liabilitiesSlaveMapper = liabilitiesSlaveMapper;
        this.liabilitiesDataSyncMapper = liabilitiesDataSyncMapper;
    }

    @Override
    @SneakyThrows
    @DSTransactional
    public Object liabilitiesSync() {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        JSONObject jsonObject = new JSONObject();
        String lastMonth = DateUtils.getYearAndMonthLast();
       //String lastMonth = "2024-09";
        ParmDataModel parmDataModel = new ParmDataModel();
        parmDataModel.setPeriod(lastMonth.replaceAll("-", ""));
        parmDataModel.setYear(lastMonth.substring(0, 4));
        String monthStr = lastMonth.substring(5, 7);
        parmDataModel.setMonth(StringUtils.startsWith(monthStr, "0") ? lastMonth.substring(6, 7) : monthStr);
        logger.info("参数:{}", JSON.toJSONString(parmDataModel));
        //先清除表内数据
        liabilitiesSlaveMapper.delete(parmDataModel);
        List<String> icpCodeList = liabilitiesSlaveMapper.getCodeList();
        List<String> stringList = new LinkedList<>();
        for (String str : icpCodeList) {
           str = str.replaceAll("[a-zA-Z]", "");
            parmDataModel.setEntity(str);
            LiabilitiesModel liabilitiesModel = liabilitiesDataSyncMapper.getLiabilities(parmDataModel);
            if (ObjectUtils.isEmpty(liabilitiesModel)) {
                stringList.add(str);
                logger.info("unexpected event {}:", "未查询到数据是："+"entityCode为" + str );
                continue;
            }
            Field[] fields = liabilitiesModel.getClass().getDeclaredFields();
            List<FreeQueryLiabilitiesModel> freeQueryLiabilitiesModelList = new LinkedList<>();
            for (Field field : fields) {
                String indexname = field.getName();
                if (StringUtils.startsWith(indexname, "ZB")) {
                    List<IndexAccountModel> indexAccountModelList = liabilitiesSlaveMapper.getAccount(indexname);
                    if (CollectionUtils.isEmpty(indexAccountModelList)) {
                        continue;
                    }
                    for (IndexAccountModel indexAccountModel : indexAccountModelList) {
                        FreeQueryLiabilitiesModel freeQueryLiabilitiesModel = new FreeQueryLiabilitiesModel();
                        Field declaredField = liabilitiesModel.getClass().getDeclaredField(indexname);
                        declaredField.setAccessible(true);
                        Object o = declaredField.get(liabilitiesModel);
                        if (o == null) {
                            continue;
                        } else {
                            freeQueryLiabilitiesModel.setMoney(StringUtils.contains((String) o, "E") ? "0" : ((String) o));
                        }
                        freeQueryLiabilitiesModel.setIndexname(field.getName());
                        freeQueryLiabilitiesModel.setAccountcode(indexAccountModel.getAccount());
                        freeQueryLiabilitiesModel.setAccountname(indexAccountModel.getIndexname());
                        freeQueryLiabilitiesModel.setEntity(str);
                        freeQueryLiabilitiesModel.setYearm(parmDataModel.getPeriod());
                        freeQueryLiabilitiesModelList.add(freeQueryLiabilitiesModel);
                    }
                }
            }
            if (!CollectionUtils.isEmpty(freeQueryLiabilitiesModelList)) {
                liabilitiesSlaveMapper.save(freeQueryLiabilitiesModelList);
            }
        }
        jsonObject.put("description", "未查询到数据是：" + String.join(",", stringList) );
        jsonObject.put("msg", "数据同步成功!");
        jsonObject.put("code", 200);
        stopWatch.stop();
        logger.info("执行时间:{}", stopWatch.getLastTaskTimeMillis());
        return jsonObject;
    }
}
